package it.telecomitalia.tc.se.cw.esb;

import it.telecomitalia.tc.se.cw.esb.exception.EsbException;
import it.telecomitalia.tc.se.cw.esb.util.Configuration;
import it.telecomitalia.tc.se.cw.esb.util.ConfigurationFactory;
import it.telecomitalia.tc.se.cw.esb.util.Constants;
import it.telecomitalia.tc.se.cw.esb.util.MarshallerUtility;
import it.telecomitalia.tc.se.schema.request.Request;

import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;
import org.jboss.soa.esb.actions.AbstractActionLifecycle;
import org.jboss.soa.esb.actions.ActionProcessingException;
import org.jboss.soa.esb.helpers.ConfigTree;
import org.jboss.soa.esb.message.Message;
import org.jbpm.JbpmConfiguration;
import org.jbpm.JbpmContext;
import org.jbpm.db.GraphSession;
import org.jbpm.graph.def.ProcessDefinition;
import org.jbpm.graph.exe.ProcessInstance;

/**
 * @Title: CwProcessMgmtWsListenerAction
 * @Description: 
 * @Copyright: Copyright (c) 2011
 * @Company: Telecomitalia
 * @author ferdinando.cascone
 */
public class CwProcessMgmtWsListenerAction extends AbstractActionLifecycle
{
	private final static Logger log= LogManager.getLogger(CwProcessMgmtWsListenerAction.class);
	
	private Configuration conf = null;
	
	protected JbpmConfiguration jbpmConfiguration=null;
	
    protected ConfigTree _config;
    
    
    public CwProcessMgmtWsListenerAction(ConfigTree config)
    {
        _config = config;
    }
    
    public Message noOperation(Message message) { return message; } 
	  
	  public Message process(Message message) throws Exception{
		    log.info("---------------------------");
		  	log.info("* The default method called");
		  	log.info("---------------------------");
	    	return message;
	  }
    
    public Message initProcess(Message message) throws ActionProcessingException {
    	log.info("-----------------------------------------------------");
		log.info("* CwProcessAction initProcess - start");
		String content = (String) message.getBody().get();
		
		log.info("* Body: " + content);
		if (content == null || content.trim().equals("")) {
			throw new ActionProcessingException(new EsbException(
					Constants.ERR_CODE_GENERIC, "Message input not valorized")); 
			}
		
		inizialize();
    	
		
		try {
			it.telecomitalia.tc.se.schema.request.Request request = MarshallerUtility
					.unmarshallerRequest(content);

			if (request.getHeader().getActor() == null
					|| request.getHeader().getActor().trim().equals("")) {
				throw new EsbException(Constants.ERR_CODE_GENERIC,
						"Actor not valorized");
			}
			
			if (request.getHeader().getActor() == null
					|| request.getHeader().getAction().trim().equals("")) {
				throw new EsbException(Constants.ERR_CODE_GENERIC,
						"Action not valorized");
			}
			
			
			String idProcess="";
			
			if(request.getHeader().getAction().equals(conf.getMandatorySetting(Constants.ACTION_PROCESS_START))){
				idProcess = startNewProcessInstance(request);
				log.info("Process id: " + idProcess);
				message.getBody().add("instanceId", idProcess);
			}else{
				idProcess="";
			}

		} catch (EsbException e) {
			e.printStackTrace();
			throw new ActionProcessingException(e.getErrorDescription(), e);
		}

		// message.getBody().add(new String("id process"));

		log.info("* CwProcessAction initProcess - end ");
		log.info("----------------------------------------------------");

		return message;
	}
    
    public String startNewProcessInstance(Request request) throws EsbException{
		log.info("---------------------------------");
		log.info("* startNewProcessInstance - start");
		log.info("*");
		log.info("* get new instance of jbpmConfiguration");
		jbpmConfiguration = JbpmConfiguration.getInstance();
		log.info("*");
		JbpmContext jbpmContext = jbpmConfiguration.createJbpmContext();
		log.info("* Creating new instance process ... ");

		String processDefinitionName = conf
				.getMandatorySetting(Constants.PROCESS_NAME);
		log.info("* Name processDefinition: " + processDefinitionName);
		
		try{
			if(!request.getHeader().getActor().equals(conf.getMandatorySetting(Constants.PROFILE_RESPONSABILE_DI_PROGETTO)))
				throw new EsbException(Constants.ERR_CODE_GENERIC,"Invalid profile for start process");
			GraphSession graphSession = jbpmContext.getGraphSession();   
			ProcessDefinition definition = graphSession.findLatestProcessDefinition(processDefinitionName);
			ProcessInstance instance = definition.createProcessInstance();
			System.out.println("Instance process: "+instance.getContextInstance().getProcessInstance().toString());
			Long id = Long.valueOf(instance.getId());
			
			/* sr id e id processo*/
			log.info("------------------------------------------------");	
			log.info("Actor: "+request.getHeader().getActor()+"\tIdProces: "+id);
			log.info("------------------------------------------------");
			if(request!=null){
				log.info("* Setting Jbpm parameter ....");
				java.util.Map<String, String> map = new java.util.HashMap<String, String>();
				
				if(!request.getParameter().isEmpty()){
					for(it.telecomitalia.tc.se.schema.request.ParameterType parameter : request.getParameter()){						
						map.put(parameter.getName(), parameter.getValue());
						log.info("name: "+parameter.getName());
						log.info("value: "+parameter.getValue());
					}
					
				}
				
				instance.getContextInstance().setVariables(map );
	      		log.info("* Setting Jbpm parameter completed ");
			}
			
			log.info("* Exectuing signal.... ");
			instance.signal();
			log.info("* Signal complete successfully !");
			
			log.info("* save  jbpmContext");
	        jbpmContext.save(instance); 
	        log.info("* startNewProcessInstance - end");
	 		  log.info("-------------------------------"); 
	 		  
	 		  return Long.toString(id);

			
		}catch(EsbException e ){
			throw(e);
			
		}
		
		
	}	
     
    /**
	 * Load properties
	 */
	private void inizialize() {
		try {
			conf = ConfigurationFactory
					.instance(Constants.FRONT_END_PROPERTIES_FILE_NAME);
		} catch (it.telecomitalia.tc.se.cw.esb.exception.ConfigurationException e1) {
			log.info("error: " + e1);
		}
	}
}
